mod atlasset;

use atlasset::AtlasSet;

use anyhow::Result;
use image::io::Reader;
use std::path::PathBuf;
use walkdir::WalkDir;

// TODO: procedural sun coloring
// TODO: transparency buffer
// TODO: don't re-encode. Direct to gpu?
// (maybe not, tiling is slow. Make it work with files first.)
// TODO: path for atlas files
// TODO: rework texturearray
// TODO: reasonable sprite sizes (especially ui, document rules)
// TODO: consistent naming
// TODO: dynamic packing (for plugins)
// spriteatlas: the set of textures
// texture: a single plane of many images, what we load to wgpu
// image: a single file
// sprite: a possibly animated texture

fn main() -> Result<()> {
	let mut files = Vec::new();

	for e in WalkDir::new("./assets/render")
		.into_iter()
		.filter_map(|e| e.ok())
	{
		if e.metadata().unwrap().is_file() {
			// TODO: better warnings
			match e.path().extension() {
				Some(t) => {
					if t.to_str() != Some("png") {
						println!("[WARNING] {e:#?} is not a png file, skipping.");
						continue;
					}
				}
				None => {
					println!("[WARNING] {e:#?} is not a png file, skipping.");
					continue;
				}
			}

			let path = e.path().to_path_buf();
			let reader = Reader::open(&path)?;
			let dim = reader.into_dimensions()?;
			files.push((path, [dim.0, dim.1]))
		}
	}

	// Sort by image size
	files.sort_by(|a, b| {
		let a = a.1[0] * a.1[1];
		let b = b.1[0] * b.1[1];
		b.cmp(&a)
	});

	// Create atlas set
	let mut atlas_set = AtlasSet::new(8192, 8192, 16);
	let total = files.len();
	let mut i = 0;
	for (path, dim) in files {
		i += 1;
		let atlas_idx = atlas_set.write_image(&path, dim)?;
		println!(
			"({i} of {total}, atlas {atlas_idx}, efficiency {:.02}%) Added {}",
			100.0 * atlas_set.get_efficiency(),
			path.display()
		);
	}

	println!(
		"Packing efficiency: {:.02}%",
		100.0 * atlas_set.get_efficiency()
	);

	println!("Saving files...");
	atlas_set.save_files(
		|x| PathBuf::from(format!("atlas-{x:0.2}.png")),
		&PathBuf::from("spriteatlas.toml"),
	)?;

	return Ok(());
}
